@@ -2,7 +2,7 @@
 
   gbx_c_array.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -599,9 +599,16 @@ BEGIN_PROPERTY(Array_Max)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Array_Empty)
+
+	GB_ReturnBoolean(THIS->count == 0);
+
+END_PROPERTY
+
+
 static bool copy_remove(CARRAY *_object, int start, int length, bool copy, bool remove)
 {
-	CARRAY *array;
+	CARRAY *array = NULL;
 	int count = THIS->count;
 	void *data;
 	int i, nsize;
@@ -612,7 +619,7 @@ static bool copy_remove(CARRAY *_object, int start, int length, bool copy, bool
 			return TRUE;
 	}
 
-	if (remove && check_not_static(THIS))
+	if (remove && check_not_static(THIS) && check_not_read_only(THIS))
 		return TRUE;
 
 	if (check_start_length(count, &start, &length))
@@ -721,7 +728,7 @@ BEGIN_METHOD(Array_Resize, GB_INTEGER size)
 
 	int size = VARG(size);
 
-	if (check_not_static(THIS) && check_not_multi(THIS))
+	if (check_not_static(THIS) && check_not_multi(THIS) && check_not_read_only(THIS))
 		return;
 
 	CARRAY_resize(THIS, size);
@@ -879,7 +886,7 @@ static void add(CARRAY *_object, GB_VALUE *value, int index)
 {
 	void *data;
 
-	if (check_not_static(THIS) && check_not_multi(THIS))
+	if (check_not_static(THIS) && check_not_multi(THIS) && check_not_read_only(THIS))
 		return;
 
 	data = insert(THIS, index);
@@ -1537,7 +1544,7 @@ BEGIN_METHOD(Array_String_join, GB_STRING sep; GB_STRING esc)
 	uint lsep = 1;
 	char *esc = "";
 	uint lesc = 0;
-	char escl, NO_WARNING(escr);
+	char escl, escr;
 	int i;
 	char **data = (char **)THIS->data;
 	char *p, *p2;
@@ -1830,7 +1837,7 @@ static void array_of_struct_put(CARRAY *_object, void *data, void *object)
 			THROW(E_ILLEGAL);
 		
 		VALUE_class_read(desc->variable.class, &temp, (void *)addr, ctype, object);
-		BORROW(&temp);
+		//BORROW(&temp);
 		addr = (char *)data + desc->variable.offset;
 		VALUE_write(&temp, (void *)addr, desc->variable.type);
 		RELEASE(&temp);
@@ -1991,6 +1998,7 @@ GB_DESC NATIVE_Array[] =
 
 	GB_PROPERTY_READ("Type", "i", Array_Type),
 	GB_PROPERTY_READ("Count", "i", Array_Count),
+	GB_PROPERTY_READ("Empty", "b", Array_Empty),
 	GB_PROPERTY_READ("Max", "i", Array_Max),
 	GB_PROPERTY_READ("Length", "i", Array_Count),
 	GB_PROPERTY_READ("Dim", "i", Array_Dim),
